@@ -53,7 +53,7 @@ extern "C" {
 typedef struct juice_agent juice_agent_t;
 
 typedef enum juice_state {
-	JUICE_STATE_DISCONNECTED,
+	JUICE_STATE_DISCONNECTED = 0,
 	JUICE_STATE_GATHERING,
 	JUICE_STATE_CONNECTING,
 	JUICE_STATE_CONNECTED,
@@ -74,7 +74,15 @@ typedef struct juice_turn_server {
 	uint16_t port;
 } juice_turn_server_t;
 
+typedef enum juice_concurrency_mode {
+	JUICE_CONCURRENCY_MODE_POLL = 0, // Connections share a single thread
+	JUICE_CONCURRENCY_MODE_MUX,      // Connections are multiplexed on a single UDP socket
+	JUICE_CONCURRENCY_MODE_THREAD,   // Each connection runs in its own thread
+} juice_concurrency_mode_t;
+
 typedef struct juice_config {
+	juice_concurrency_mode_t concurrency_mode;
+
 	const char *stun_server_host;
 	uint16_t stun_server_port;
 
@@ -150,8 +158,8 @@ JUICE_EXPORT int juice_server_add_credentials(juice_server_t *server,
 
 // Logging
 
-typedef enum {
-	JUICE_LOG_LEVEL_VERBOSE,
+typedef enum juice_log_level {
+	JUICE_LOG_LEVEL_VERBOSE = 0,
 	JUICE_LOG_LEVEL_DEBUG,
 	JUICE_LOG_LEVEL_INFO,
 	JUICE_LOG_LEVEL_WARN,
