@@ -141,14 +141,21 @@ juice_agent_t *agent_create(const juice_config_t *config) {
 	return agent;
 
 error:
-	agent_do_destroy(agent);
+	agent_destroy(agent);
 	return NULL;
 }
 
-void agent_do_destroy(juice_agent_t *agent) {
+void agent_destroy(juice_agent_t *agent) {
 	JLOG_DEBUG("Destroying agent");
 
-	conn_destroy(agent);
+	if (agent->resolver_thread_started) {
+		JLOG_VERBOSE("Waiting for resolver thread");
+		thread_join(agent->resolver_thread, NULL);
+	}
+
+	if (agent->conn_impl) {
+		conn_destroy(agent);
+	}
 
 	// Free credentials in entries
 	for (int i = 0; i < agent->entries_count; ++i) {
@@ -174,12 +181,26 @@ void agent_do_destroy(juice_agent_t *agent) {
 #ifdef _WIN32
 	WSACleanup();
 #endif
+
 	JLOG_VERBOSE("Destroyed agent");
 }
 
-void agent_destroy(juice_agent_t *agent) {
-	// TODO
-	agent_do_destroy(agent);
+static bool has_nonnumeric_server_hostnames(const juice_config_t *config) {
+	if (config->stun_server_host && !addr_is_numeric_hostname(config->stun_server_host))
+		return true;
+
+	for (int i = 0; i < config->turn_servers_count; ++i) {
+		juice_turn_server_t *turn_server = config->turn_servers + i;
+		if (turn_server->host && !addr_is_numeric_hostname(turn_server->host))
+			return true;
+	}
+
+	return false;
+}
+
+static thread_return_t THREAD_CALL resolver_thread_entry(void *arg) {
+	agent_resolve_servers((juice_agent_t *)arg);
+	return (thread_return_t)0;
 }
 
 int agent_gather_candidates(juice_agent_t *agent) {
@@ -200,6 +221,11 @@ int agent_gather_candidates(juice_agent_t *agent) {
 		return -1;
 	}
 
+	if (agent->mode == AGENT_MODE_UNKNOWN) {
+		JLOG_DEBUG("Assuming controlling mode");
+		agent->mode = AGENT_MODE_CONTROLLING;
+	}
+
 	addr_record_t records[ICE_MAX_CANDIDATES_COUNT - 1];
 	int records_count = conn_get_addrs(agent, records, ICE_MAX_CANDIDATES_COUNT - 1);
 	if (records_count < 0) {
@@ -249,12 +275,31 @@ int agent_gather_candidates(juice_agent_t *agent) {
 			agent->config.cb_candidate(agent, buffer, agent->config.user_ptr);
 	}
 
-	if (agent->mode == AGENT_MODE_UNKNOWN) {
-		JLOG_DEBUG("Assuming controlling mode");
-		agent->mode = AGENT_MODE_CONTROLLING;
+	agent_change_state(agent, JUICE_STATE_CONNECTING);
+	conn_unlock(agent);
+	conn_interrupt(agent);
+
+	if (has_nonnumeric_server_hostnames(&agent->config)) {
+		// Resolve server hostnames in a separate thread as it may block
+		JLOG_DEBUG("Starting resolver thread for servers");
+		int ret = thread_init(&agent->resolver_thread, resolver_thread_entry, agent);
+		if (ret) {
+			JLOG_FATAL("Thread creation failed, error=%d", ret);
+			agent_update_gathering_done(agent);
+			return -1;
+		}
+		agent->resolver_thread_started = true;
+	} else {
+		JLOG_DEBUG("Resolving servers synchronously");
+		if (agent_resolve_servers(agent) < 0)
+			return -1;
 	}
 
-	agent_change_state(agent, JUICE_STATE_CONNECTING);
+	return 0;
+}
+
+int agent_resolve_servers(juice_agent_t *agent) {
+	conn_lock(agent);
 
 	// TURN server resolution
 	juice_concurrency_mode_t mode = agent->config.concurrency_mode;
@@ -269,12 +314,14 @@ int agent_gather_candidates(juice_agent_t *agent) {
 				break;
 
 			juice_turn_server_t *turn_server = agent->config.turn_servers + i;
+			if (!turn_server->host)
+				continue;
+
 			if (!turn_server->port)
 				turn_server->port = 3478; // default TURN port
 
 			char service[8];
 			snprintf(service, 8, "%hu", turn_server->port);
-
 			addr_record_t records[DEFAULT_MAX_RECORDS_COUNT];
 			int records_count =
 			    addr_resolve(turn_server->host, service, records, DEFAULT_MAX_RECORDS_COUNT);
@@ -367,7 +414,7 @@ int agent_gather_candidates(juice_agent_t *agent) {
 
 	agent_update_gathering_done(agent);
 	conn_unlock(agent);
-	conn_interrupt(agent); // TODO: would not be necessary if created in locked state
+	conn_interrupt(agent);
 	return 0;
 }
 
@@ -409,6 +456,12 @@ int agent_set_remote_description(juice_agent_t *agent, const char *sdp) {
 		conn_unlock(agent);
 		return -1;
 	}
+
+	if (agent->mode == AGENT_MODE_UNKNOWN) {
+		JLOG_DEBUG("Assuming controlled mode");
+		agent->mode = AGENT_MODE_CONTROLLED;
+	}
+
 	// There is only one component, therefore we can unfreeze already existing pairs now
 	JLOG_DEBUG("Unfreezing %d existing candidate pairs", (int)agent->candidate_pairs_count);
 	for (int i = 0; i < agent->candidate_pairs_count; ++i) {
@@ -420,10 +473,6 @@ int agent_set_remote_description(juice_agent_t *agent, const char *sdp) {
 		if (agent_add_candidate_pairs_for_remote(agent, remote))
 			JLOG_WARN("Failed to add candidate pair from remote description");
 	}
-	if (agent->mode == AGENT_MODE_UNKNOWN) {
-		JLOG_DEBUG("Assuming controlled mode");
-		agent->mode = AGENT_MODE_CONTROLLED;
-	}
 
 	conn_unlock(agent);
 	conn_interrupt(agent);
@@ -466,7 +515,7 @@ int agent_set_remote_gathering_done(juice_agent_t *agent) {
 }
 
 int agent_send(juice_agent_t *agent, const char *data, size_t size, int ds) {
-	// Try not to lock the global mutex in the send path
+	// Try not to lock in the send path
 	agent_stun_entry_t *selected_entry = atomic_load(&agent->selected_entry);
 	if (!selected_entry) {
 		JLOG_ERROR("Send called before ICE is connected");
@@ -499,7 +548,7 @@ int agent_relay_send(juice_agent_t *agent, agent_stun_entry_t *entry, const addr
 		return -1;
 	}
 
-	JLOG_VERBOSE("Sending datagram via relay, size=%d", size);
+	JLOG_VERBOSE("Sending datagram via TURN Send Indication, size=%d", size);
 
 	// Send CreatePermission if necessary
 	if (!turn_has_permission(&entry->turn->map, dst))
@@ -539,7 +588,7 @@ int agent_channel_send(juice_agent_t *agent, agent_stun_entry_t *entry, const ad
 		if (agent_send_turn_channel_bind_request(agent, entry, record, ds, &channel) < 0)
 			return -1;
 
-	JLOG_VERBOSE("Sending datagram via channel 0x%hX, size=%d", channel, size);
+	JLOG_VERBOSE("Sending datagram via TURN ChannelData, channel=0x%hX, size=%d", channel, size);
 
 	// Send the data wrapped as ChannelData
 	char buffer[BUFFER_SIZE];
@@ -761,7 +810,8 @@ int agent_bookkeeping(juice_agent_t *agent, timestamp_t *next_timestamp) {
 				JLOG_WARN("Sending keepalive failed");
 
 			agent_arm_transmission(agent, entry,
-			                       entry->type == AGENT_STUN_ENTRY_TYPE_RELAY && ret >= 0
+			                       ret >= 0 && entry->type == AGENT_STUN_ENTRY_TYPE_RELAY &&
+			                               agent->remote.candidates_count > 0
 			                           ? TURN_REFRESH_PERIOD
 			                           : STUN_KEEPALIVE_PERIOD);
 
@@ -1227,6 +1277,7 @@ int agent_process_stun_binding(juice_agent_t *agent, const stun_message_t *msg,
 				JLOG_WARN("Failed to add local peer reflexive candidate from STUN mapped address");
 			}
 		}
+
 		if (entry->type == AGENT_STUN_ENTRY_TYPE_CHECK) {
 			// 7.2.5.2.1. Non-Symmetric Transport Addresses:
 			// The ICE agent MUST check that the source and destination transport addresses in the
@@ -1287,7 +1338,7 @@ int agent_process_stun_binding(juice_agent_t *agent, const stun_message_t *msg,
 					// attribute in the request, the agent MUST switch to the controlled role. Once
 					// the agent has switched its role, the agent MUST [...] set the candidate pair
 					// state to Waiting [and] change the tiebreaker value.
-					JLOG_WARN("ICE role conflit");
+					JLOG_WARN("ICE role conflict");
 					JLOG_DEBUG("Switching roles to %s as requested",
 					           entry->mode == AGENT_MODE_CONTROLLING ? "controlled"
 					                                                 : "controlling");
@@ -1422,7 +1473,13 @@ int agent_send_stun_binding(juice_agent_t *agent, agent_stun_entry_t *entry, stu
 	if (entry->relay_entry) {
 		// The datagram must be sent through the relay
 		JLOG_DEBUG("Sending STUN message via relay");
-		if (agent_relay_send(agent, entry->relay_entry, &entry->record, buffer, size, 0) < 0) {
+		int ret;
+		if (agent->state == JUICE_STATE_COMPLETED)
+			ret = agent_channel_send(agent, entry->relay_entry, &entry->record, buffer, size, 0);
+		else
+			ret = agent_relay_send(agent, entry->relay_entry, &entry->record, buffer, size, 0);
+
+		if (ret < 0) {
 			JLOG_WARN("STUN message send via relay failed");
 			return -1;
 		}
@@ -1462,11 +1519,21 @@ int agent_process_turn_allocate(juice_agent_t *agent, const stun_message_t *msg,
 			// There is nothing to do other than rearm
 			if (entry->state == AGENT_STUN_ENTRY_STATE_SUCCEEDED_KEEPALIVE) {
 				juice_random(entry->transaction_id, STUN_TRANSACTION_ID_SIZE);
-				agent_arm_transmission(agent, entry, TURN_REFRESH_PERIOD);
+				agent_arm_transmission(agent, entry,
+				                       agent->remote.candidates_count > 0 ? TURN_REFRESH_PERIOD
+				                                                          : STUN_KEEPALIVE_PERIOD);
 			}
 			break;
 		}
 
+		JLOG_DEBUG("TURN allocate successful");
+
+		if (!msg->relayed.len) {
+			JLOG_ERROR("Expected relayed address in TURN Allocate response");
+			entry->state = AGENT_STUN_ENTRY_STATE_FAILED;
+			return -1;
+		}
+
 		if (entry->state != AGENT_STUN_ENTRY_STATE_SUCCEEDED_KEEPALIVE) {
 			entry->state = AGENT_STUN_ENTRY_STATE_SUCCEEDED;
 			entry->next_transmission = 0;
@@ -1476,7 +1543,9 @@ int agent_process_turn_allocate(juice_agent_t *agent, const stun_message_t *msg,
 			// We want to send refresh requests now
 			entry->state = AGENT_STUN_ENTRY_STATE_SUCCEEDED_KEEPALIVE;
 			juice_random(entry->transaction_id, STUN_TRANSACTION_ID_SIZE);
-			agent_arm_transmission(agent, entry, TURN_REFRESH_PERIOD);
+			agent_arm_transmission(agent, entry,
+			                       agent->remote.candidates_count > 0 ? TURN_REFRESH_PERIOD
+			                                                          : STUN_KEEPALIVE_PERIOD);
 		}
 
 		if (msg->mapped.len) {
@@ -1494,13 +1563,6 @@ int agent_process_turn_allocate(juice_agent_t *agent, const stun_message_t *msg,
 			}
 		}
 
-		if (!msg->relayed.len) {
-			JLOG_ERROR("Expected relayed address in TURN %s response",
-			           msg->msg_method == STUN_METHOD_ALLOCATE ? "Allocate" : "Refresh");
-			entry->state = AGENT_STUN_ENTRY_STATE_FAILED;
-			return -1;
-		}
-
 		entry->relayed = msg->relayed;
 		if (agent_add_local_relayed_candidate(agent, &msg->relayed)) {
 			JLOG_WARN("Failed to add local relayed candidate from TURN relayed address");
