@@ -66,8 +66,13 @@ using std::variant;
 using std::weak_ptr;
 
 using binary = std::vector<byte>;
-using binary_ptr = std::shared_ptr<binary>;
+using binary_ptr = shared_ptr<binary>;
+using message_variant = variant<binary, string>;
 
+using std::int16_t;
+using std::int32_t;
+using std::int64_t;
+using std::int8_t;
 using std::ptrdiff_t;
 using std::size_t;
 using std::uint16_t;
